@@ -1,4 +1,4 @@
-/*	$OpenBSD: cert.c,v 1.107 2023/04/15 00:39:08 job Exp $ */
+/*	$OpenBSD: cert.c,v 1.114 2023/06/29 10:28:25 tb Exp $ */
 /*
  * Copyright (c) 2022 Theo Buehler <tb@openbsd.org>
  * Copyright (c) 2021 Job Snijders <job@openbsd.org>
@@ -167,13 +167,13 @@ sbgp_assysnum(struct parse *p, X509_EXTENSION *ext)
 	int			 i, rc = 0;
 
 	if (!X509_EXTENSION_get_critical(ext)) {
-		cryptowarnx("%s: RFC 6487 section 4.8.11: autonomousSysNum: "
+		warnx("%s: RFC 6487 section 4.8.11: autonomousSysNum: "
 		    "extension not critical", p->fn);
 		goto out;
 	}
 
 	if ((asidentifiers = X509V3_EXT_d2i(ext)) == NULL) {
-		cryptowarnx("%s: RFC 6487 section 4.8.11: autonomousSysNum: "
+		warnx("%s: RFC 6487 section 4.8.11: autonomousSysNum: "
 		    "failed extension parse", p->fn);
 		goto out;
 	}
@@ -204,6 +204,11 @@ sbgp_assysnum(struct parse *p, X509_EXTENSION *ext)
 		goto out;
 	}
 
+	if (asz == 0) {
+		warnx("%s: RFC 6487 section 4.8.11: empty asIdsOrRanges",
+		    p->fn);
+		goto out;
+	}
 	if (asz >= MAX_AS_SIZE) {
 		warnx("%s: too many AS number entries: limit %d",
 		    p->fn, MAX_AS_SIZE);
@@ -343,13 +348,13 @@ sbgp_ipaddrblk(struct parse *p, X509_EXTENSION *ext)
 	int				 i, j, rc = 0;
 
 	if (!X509_EXTENSION_get_critical(ext)) {
-		cryptowarnx("%s: RFC 6487 section 4.8.10: sbgp-ipAddrBlock: "
+		warnx("%s: RFC 6487 section 4.8.10: sbgp-ipAddrBlock: "
 		    "extension not critical", p->fn);
 		goto out;
 	}
 
 	if ((addrblk = X509V3_EXT_d2i(ext)) == NULL) {
-		cryptowarnx("%s: RFC 6487 section 4.8.10: sbgp-ipAddrBlock: "
+		warnx("%s: RFC 6487 section 4.8.10: sbgp-ipAddrBlock: "
 		    "failed extension parse", p->fn);
 		goto out;
 	}
@@ -371,6 +376,11 @@ sbgp_ipaddrblk(struct parse *p, X509_EXTENSION *ext)
 			    p->fn, af->ipAddressChoice->type);
 			goto out;
 		}
+		if (ipsz == p->res->ipsz) {
+			warnx("%s: RFC 6487 section 4.8.10: "
+			    "empty ipAddressesOrRanges", p->fn);
+			goto out;
+		}
 
 		if (ipsz >= MAX_IP_SIZE)
 			goto out;
@@ -412,6 +422,11 @@ sbgp_ipaddrblk(struct parse *p, X509_EXTENSION *ext)
 		}
 	}
 
+	if (p->res->ipsz == 0) {
+		warnx("%s: RFC 6487 section 4.8.10: empty ipAddrBlock", p->fn);
+		goto out;
+	}
+
 	rc = 1;
  out:
 	sk_IPAddressFamily_pop_free(addrblk, IPAddressFamily_free);
@@ -438,8 +453,8 @@ sbgp_sia(struct parse *p, X509_EXTENSION *ext)
 	}
 
 	if ((sia = X509V3_EXT_d2i(ext)) == NULL) {
-		cryptowarnx("%s: RFC 6487 section 4.8.8: SIA: "
-		    "failed extension parse", p->fn);
+		warnx("%s: RFC 6487 section 4.8.8: SIA: failed extension parse",
+		    p->fn);
 		goto out;
 	}
 
@@ -515,13 +530,13 @@ certificate_policies(struct parse *p, X509_EXTENSION *ext)
 	int				 rc = 0;
 
 	if (!X509_EXTENSION_get_critical(ext)) {
-		cryptowarnx("%s: RFC 6487 section 4.8.9: certificatePolicies: "
+		warnx("%s: RFC 6487 section 4.8.9: certificatePolicies: "
 		    "extension not critical", p->fn);
 		goto out;
 	}
 
 	if ((policies = X509V3_EXT_d2i(ext)) == NULL) {
-		cryptowarnx("%s: RFC 6487 section 4.8.9: certificatePolicies: "
+		warnx("%s: RFC 6487 section 4.8.9: certificatePolicies: "
 		    "failed extension parse", p->fn);
 		goto out;
 	}
@@ -596,6 +611,11 @@ cert_parse_ee_cert(const char *fn, X509 *x)
 	if ((p.res = calloc(1, sizeof(struct cert))) == NULL)
 		err(1, NULL);
 
+	if (X509_get_version(x) != 2) {
+		warnx("%s: RFC 6487 4.1: X.509 version must be v3", fn);
+		goto out;
+	}
+
 	if (X509_get_key_usage(x) != KU_DIGITAL_SIGNATURE) {
 		warnx("%s: RFC 6487 section 4.8.4: KU must be digitalSignature",
 		    fn);
@@ -621,7 +641,7 @@ cert_parse_ee_cert(const char *fn, X509 *x)
 	}
 
 	if (!X509_up_ref(x)) {
-		cryptowarnx("%s: X509_up_ref failed", fn);
+		warnx("%s: X509_up_ref failed", fn);
 		goto out;
 	}
 
@@ -643,7 +663,6 @@ cert_parse_pre(const char *fn, const unsigned char *der, size_t len)
 {
 	const unsigned char	*oder;
 	int			 extsz;
-	int			 sia_present = 0;
 	size_t			 i;
 	X509			*x = NULL;
 	X509_EXTENSION		*ext = NULL;
@@ -653,7 +672,10 @@ cert_parse_pre(const char *fn, const unsigned char *der, size_t len)
 	ASN1_OBJECT		*obj;
 	EVP_PKEY		*pkey;
 	struct parse		 p;
-	int			 nid;
+	int			 nid, ip, as, sia, cp, crldp, aia, aki, ski,
+				 eku, bc, ku;
+
+	nid = ip = as = sia = cp = crldp = aia = aki = ski = eku = bc = ku = 0;
 
 	/* just fail for empty buffers, the warning was printed elsewhere */
 	if (der == NULL)
@@ -666,7 +688,7 @@ cert_parse_pre(const char *fn, const unsigned char *der, size_t len)
 
 	oder = der;
 	if ((x = d2i_X509(NULL, &der, len)) == NULL) {
-		cryptowarnx("%s: d2i_X509", p.fn);
+		warnx("%s: d2i_X509", p.fn);
 		goto out;
 	}
 	if (der != oder + len) {
@@ -676,13 +698,18 @@ cert_parse_pre(const char *fn, const unsigned char *der, size_t len)
 
 	/* Cache X509v3 extensions, see X509_check_ca(3). */
 	if (X509_check_purpose(x, -1, -1) <= 0) {
-		cryptowarnx("%s: could not cache X509v3 extensions", p.fn);
+		warnx("%s: could not cache X509v3 extensions", p.fn);
+		goto out;
+	}
+
+	if (X509_get_version(x) != 2) {
+		warnx("%s: RFC 6487 4.1: X.509 version must be v3", fn);
 		goto out;
 	}
 
 	X509_get0_signature(NULL, &palg, x);
 	if (palg == NULL) {
-		cryptowarnx("%s: X509_get0_signature", p.fn);
+		warnx("%s: X509_get0_signature", p.fn);
 		goto out;
 	}
 	X509_ALGOR_get0(&cobj, NULL, NULL, palg);
@@ -703,7 +730,7 @@ cert_parse_pre(const char *fn, const unsigned char *der, size_t len)
 	/* Look for X509v3 extensions. */
 
 	if ((extsz = X509_get_ext_count(x)) < 0)
-		cryptoerrx("X509_get_ext_count");
+		errx(1, "X509_get_ext_count");
 
 	for (i = 0; i < (size_t)extsz; i++) {
 		ext = X509_get_ext(x, i);
@@ -711,38 +738,58 @@ cert_parse_pre(const char *fn, const unsigned char *der, size_t len)
 		obj = X509_EXTENSION_get_object(ext);
 		assert(obj != NULL);
 
-		switch (OBJ_obj2nid(obj)) {
+		switch (nid = OBJ_obj2nid(obj)) {
 		case NID_sbgp_ipAddrBlock:
+			if (ip++ > 0)
+				goto dup;
 			if (!sbgp_ipaddrblk(&p, ext))
 				goto out;
 			break;
 		case NID_sbgp_autonomousSysNum:
+			if (as++ > 0)
+				goto dup;
 			if (!sbgp_assysnum(&p, ext))
 				goto out;
 			break;
 		case NID_sinfo_access:
-			sia_present = 1;
+			if (sia++ > 0)
+				goto dup;
 			if (!sbgp_sia(&p, ext))
 				goto out;
 			break;
 		case NID_certificate_policies:
+			if (cp++ > 0)
+				goto dup;
 			if (!certificate_policies(&p, ext))
 				goto out;
 			break;
 		case NID_crl_distribution_points:
-			/* ignored here, handled later */
+			if (crldp++ > 0)
+				goto dup;
 			break;
 		case NID_info_access:
+			if (aia++ > 0)
+				goto dup;
 			break;
 		case NID_authority_key_identifier:
+			if (aki++ > 0)
+				goto dup;
 			break;
 		case NID_subject_key_identifier:
+			if (ski++ > 0)
+				goto dup;
 			break;
 		case NID_ext_key_usage:
+			if (eku++ > 0)
+				goto dup;
 			break;
 		case NID_basic_constraints:
+			if (bc++ > 0)
+				goto dup;
 			break;
 		case NID_key_usage:
+			if (ku++ > 0)
+				goto dup;
 			break;
 		default:
 			/* unexpected extensions warrant investigation */
@@ -821,7 +868,7 @@ cert_parse_pre(const char *fn, const unsigned char *der, size_t len)
 				goto out;
 			}
 		}
-		if (sia_present) {
+		if (sia) {
 			warnx("%s: unexpected SIA extension in BGPsec cert",
 			    p.fn);
 			goto out;
@@ -840,7 +887,10 @@ cert_parse_pre(const char *fn, const unsigned char *der, size_t len)
 	p.res->x509 = x;
 	return p.res;
 
-out:
+ dup:
+	warnx("%s: RFC 5280 section 4.2: duplicate %s extension", fn,
+	    OBJ_nid2sn(nid));
+ out:
 	cert_free(p.res);
 	X509_free(x);
 	return NULL;
@@ -891,15 +941,15 @@ ta_parse(const char *fn, struct cert *p, const unsigned char *pkey,
 	/* first check pubkey against the one from the TAL */
 	pk = d2i_PUBKEY(NULL, &pkey, pkeysz);
 	if (pk == NULL) {
-		cryptowarnx("%s: RFC 6487 (trust anchor): bad TAL pubkey", fn);
+		warnx("%s: RFC 6487 (trust anchor): bad TAL pubkey", fn);
 		goto badcert;
 	}
 	if ((opk = X509_get0_pubkey(p->x509)) == NULL) {
-		cryptowarnx("%s: RFC 6487 (trust anchor): missing pubkey", fn);
+		warnx("%s: RFC 6487 (trust anchor): missing pubkey", fn);
 		goto badcert;
 	}
 	if (EVP_PKEY_cmp(pk, opk) != 1) {
-		cryptowarnx("%s: RFC 6487 (trust anchor): "
+		warnx("%s: RFC 6487 (trust anchor): "
 		    "pubkey does not match TAL pubkey", fn);
 		goto badcert;
 	}
@@ -1092,6 +1142,7 @@ auth_insert(struct auth_tree *auths, struct cert *cert, struct auth *parent)
 
 	na->parent = parent;
 	na->cert = cert;
+	na->any_inherits = x509_any_inherits(cert->x509);
 
 	if (RB_INSERT(auth_tree, auths, na) != NULL)
 		err(1, "auth tree corrupted");
