@@ -1,4 +1,4 @@
-/*	$OpenBSD: extern.h,v 1.180 2023/04/27 08:37:53 beck Exp $ */
+/*	$OpenBSD: extern.h,v 1.188 2023/06/29 14:33:35 tb Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -348,11 +348,7 @@ struct gbr {
 	time_t		 notbefore; /* EE cert's Not Before */
 	time_t		 notafter; /* Not After of the GBR EE */
 	time_t		 expires; /* when the signature path expires */
-};
-
-struct aspa_provider {
-	uint32_t	 as;
-	enum afi	 afi;
+	int		 talid; /* TAL the GBR is chained up to */
 };
 
 /*
@@ -366,7 +362,7 @@ struct aspa {
 	char			*sia; /* SIA signedObject */
 	char			*ski; /* SKI */
 	uint32_t		 custasid; /* the customerASID */
-	struct aspa_provider	*providers; /* the providers */
+	uint32_t		*providers; /* the providers */
 	size_t			 providersz; /* number of providers */
 	time_t			 signtime; /* CMS signing-time attribute */
 	time_t		 	 notbefore; /* EE cert's Not Before */
@@ -381,7 +377,7 @@ struct aspa {
 struct vap {
 	RB_ENTRY(vap)		 entry;
 	uint32_t		 custasid;
-	struct aspa_provider	*providers;
+	uint32_t		*providers;
 	size_t			 providersz;
 	time_t			 expires;
 	int			 talid;
@@ -454,6 +450,7 @@ struct auth {
 	RB_ENTRY(auth)	 entry;
 	struct cert	*cert; /* owner information */
 	struct auth	*parent; /* pointer to parent or NULL for TA cert */
+	int		 any_inherits;
 };
 /*
  * Tree of auth sorted by ski
@@ -484,6 +481,9 @@ enum rrdp_msg {
 	RRDP_ABORT,
 };
 
+/* Maximum number of delta files per RRDP notification file. */
+#define MAX_RRDP_DELTAS		300
+
 /*
  * RRDP session state, needed to pickup at the right spot on next run.
  */
@@ -491,6 +491,7 @@ struct rrdp_session {
 	char			*last_mod;
 	char			*session_id;
 	long long		 serial;
+	char			*deltas[MAX_RRDP_DELTAS];
 };
 
 /*
@@ -529,9 +530,7 @@ enum stype {
 	STYPE_TOTAL,
 	STYPE_UNIQUE,
 	STYPE_DEC_UNIQUE,
-	STYPE_BOTH,
-	STYPE_ONLY_IPV4,
-	STYPE_ONLY_IPV6,
+	STYPE_PROVIDERS,
 };
 
 struct repo;
@@ -561,8 +560,6 @@ struct repotalstats {
 	uint32_t	 vaps; /* total number of Validated ASPA Payloads */
 	uint32_t	 vaps_uniqs; /* total number of unique VAPs */
 	uint32_t	 vaps_pas; /* total number of providers */
-	uint32_t	 vaps_pas4; /* total number of IPv4 only providers */
-	uint32_t	 vaps_pas6; /* total number of IPv6 only providers */
 	uint32_t	 vrps; /* total number of Validated ROA Payloads */
 	uint32_t	 vrps_uniqs; /* number of unique vrps */
 };
@@ -688,7 +685,8 @@ int		 valid_origin(const char *, const char *);
 int		 valid_x509(char *, X509_STORE_CTX *, X509 *, struct auth *,
 		    struct crl *, const char **);
 int		 valid_rsc(const char *, struct cert *, struct rsc *);
-int		 valid_econtent_version(const char *, const ASN1_INTEGER *);
+int		 valid_econtent_version(const char *, const ASN1_INTEGER *,
+		    uint64_t);
 int		 valid_aspa(const char *, struct cert *, struct aspa *);
 int		 valid_geofeed(const char *, struct cert *, struct geofeed *);
 int		 valid_uuid(const char *);
@@ -754,9 +752,13 @@ void		 proc_http(char *, int) __attribute__((noreturn));
 void		 proc_rrdp(int) __attribute__((noreturn));
 
 /* Repository handling */
-int		 filepath_add(struct filepath_tree *, char *);
+int		 filepath_add(struct filepath_tree *, char *, time_t);
 void		 rrdp_clear(unsigned int);
-void		 rrdp_save_state(unsigned int, struct rrdp_session *);
+void		 rrdp_session_save(unsigned int, struct rrdp_session *);
+void		 rrdp_session_free(struct rrdp_session *);
+void		 rrdp_session_buffer(struct ibuf *,
+		    const struct rrdp_session *);
+struct rrdp_session	*rrdp_session_read(struct ibuf *);
 int		 rrdp_handle_file(unsigned int, enum publish_type, char *,
 		    char *, size_t, char *, size_t);
 char		*repo_basedir(const struct repo *, int);
@@ -793,14 +795,6 @@ void		 rrdp_fetch(unsigned int, const char *, const char *,
 void		 rrdp_abort(unsigned int);
 void		 rrdp_http_done(unsigned int, enum http_result, const char *);
 
-/* Logging (though really used for OpenSSL errors). */
-
-void		 cryptowarnx(const char *, ...)
-			__attribute__((format(printf, 1, 2)));
-void		 cryptoerrx(const char *, ...)
-			__attribute__((format(printf, 1, 2)))
-			__attribute__((noreturn));
-
 /* Encoding functions for hex and base64. */
 
 unsigned char	*load_file(const char *, size_t *);
@@ -958,15 +952,12 @@ int	mkpathat(int, const char *);
 /* Maximum number of delegated hosting locations (repositories) for each TAL. */
 #define MAX_REPO_PER_TAL	1000
 
-/* Maximum number of delta files per RRDP notification file. */
-#define MAX_RRDP_DELTAS		300
-
 /*
  * Time - Evaluation time is used as the current time if it is
  * larger than X509_TIME_MIN, otherwise the system time is used.
  */
 #define X509_TIME_MAX 253402300799LL
 #define X509_TIME_MIN -62167219200LL
-extern int64_t evaluation_time;
+extern time_t  get_current_time(void);
 
 #endif /* ! EXTERN_H */
