@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.236 2023/04/27 08:37:53 beck Exp $ */
+/*	$OpenBSD: main.c,v 1.244 2023/06/29 14:33:35 tb Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -74,7 +74,7 @@ int	rrdpon = 1;
 int	repo_timeout;
 time_t	deadline;
 
-int64_t  evaluation_time;
+int64_t  evaluation_time = X509_TIME_MIN;
 
 struct stats	 stats;
 
@@ -126,6 +126,14 @@ entity_free(struct entity *ent)
 	free(ent);
 }
 
+time_t
+get_current_time(void)
+{
+	if (evaluation_time > X509_TIME_MIN)
+		return (time_t) evaluation_time;
+	return time(NULL);
+}
+
 /*
  * Read a queue entity from the descriptor.
  * Matched by entity_buffer_req().
@@ -272,9 +280,8 @@ rrdp_fetch(unsigned int id, const char *uri, const char *local,
 	io_simple_buffer(b, &id, sizeof(id));
 	io_str_buffer(b, local);
 	io_str_buffer(b, uri);
-	io_str_buffer(b, s->session_id);
-	io_simple_buffer(b, &s->serial, sizeof(s->serial));
-	io_str_buffer(b, s->last_mod);
+
+	rrdp_session_buffer(b, s);
 	io_close_buffer(&rrdpq, b);
 }
 
@@ -333,7 +340,7 @@ http_fetch(unsigned int id, const char *uri, const char *last_mod, int fd)
 	io_str_buffer(b, uri);
 	io_str_buffer(b, last_mod);
 	/* pass file as fd */
-	b->fd = fd;
+	ibuf_fd_set(b, fd);
 	io_close_buffer(&httpq, b);
 }
 
@@ -354,7 +361,7 @@ rrdp_http_fetch(unsigned int id, const char *uri, const char *last_mod)
 	b = io_new_buffer();
 	io_simple_buffer(b, &type, sizeof(type));
 	io_simple_buffer(b, &id, sizeof(id));
-	b->fd = pi[0];
+	ibuf_fd_set(b, pi[0]);
 	io_close_buffer(&rrdpq, b);
 
 	http_fetch(id, uri, last_mod, pi[1]);
@@ -551,6 +558,7 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 	struct aspa	*aspa;
 	struct repo	*rp;
 	char		*file;
+	time_t		 mtime;
 	unsigned int	 id;
 	int		 talid;
 	int		 c;
@@ -565,12 +573,13 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 	io_read_buf(b, &id, sizeof(id));
 	io_read_buf(b, &talid, sizeof(talid));
 	io_read_str(b, &file);
+	io_read_buf(b, &mtime, sizeof(mtime));
 
 	/* in filemode messages can be ignored, only the accounting matters */
 	if (filemode)
 		goto done;
 
-	if (filepath_add(&fpt, file) == 0) {
+	if (filepath_add(&fpt, file, mtime) == 0) {
 		warnx("%s: File already visited", file);
 		goto done;
 	}
@@ -669,7 +678,7 @@ rrdp_process(struct ibuf *b)
 {
 	enum rrdp_msg type;
 	enum publish_type pt;
-	struct rrdp_session s;
+	struct rrdp_session *s;
 	char *uri, *last_mod, *data;
 	char hash[SHA256_DIGEST_LENGTH];
 	size_t dsz;
@@ -690,12 +699,9 @@ rrdp_process(struct ibuf *b)
 		rrdp_http_fetch(id, uri, last_mod);
 		break;
 	case RRDP_SESSION:
-		io_read_str(b, &s.session_id);
-		io_read_buf(b, &s.serial, sizeof(s.serial));
-		io_read_str(b, &s.last_mod);
-		rrdp_save_state(id, &s);
-		free(s.session_id);
-		free(s.last_mod);
+		s = rrdp_session_read(b);
+		rrdp_session_save(id, s);
+		rrdp_session_free(s);
 		break;
 	case RRDP_FILE:
 		io_read_buf(b, &pt, sizeof(pt));
@@ -744,8 +750,6 @@ sum_stats(const struct repo *rp, const struct repotalstats *in, void *arg)
 	out->vaps += in->vaps;
 	out->vaps_uniqs += in->vaps_uniqs;
 	out->vaps_pas += in->vaps_pas;
-	out->vaps_pas4 += in->vaps_pas4;
-	out->vaps_pas6 += in->vaps_pas6;
 }
 
 static void
@@ -755,6 +759,7 @@ sum_repostats(const struct repo *rp, const struct repostats *in, void *arg)
 
 	out->del_files += in->del_files;
 	out->extra_files += in->extra_files;
+	out->del_extra_files += in->del_extra_files;
 	out->del_dirs += in->del_dirs;
 	timespecadd(&in->sync_time, &out->sync_time, &out->sync_time);
 }
@@ -965,8 +970,6 @@ main(int argc, char *argv[])
 	    "proc exec unveil", NULL) == -1)
 		err(1, "pledge");
 
-	evaluation_time = time(NULL);
-
 	while ((c = getopt(argc, argv, "Ab:Bcd:e:fH:jmnoP:rRs:S:t:T:vV")) != -1)
 		switch (c) {
 		case 'A':
@@ -1008,7 +1011,7 @@ main(int argc, char *argv[])
 			outformats |= FORMAT_OPENBGPD;
 			break;
 		case 'P':
-			evaluation_time = strtonum(optarg, X509_TIME_MIN,
+			evaluation_time = strtonum(optarg, X509_TIME_MIN + 1,
 			    X509_TIME_MAX, &errs);
 			if (errs)
 				errx(1, "-P: time in seconds %s", errs);
@@ -1451,9 +1454,10 @@ main(int argc, char *argv[])
 	printf("Ghostbuster records: %u\n", stats.repo_tal_stats.gbrs);
 	printf("Trust Anchor Keys: %u\n", stats.repo_tal_stats.taks);
 	printf("Repositories: %u\n", stats.repos);
-	printf("Cleanup: removed %u files, %u directories, %u superfluous\n",
+	printf("Cleanup: removed %u files, %u directories\n"
+	    "Repository cleanup: kept %u and removed %u superfluous files\n",
 	    stats.repo_stats.del_files, stats.repo_stats.del_dirs,
-	    stats.repo_stats.extra_files);
+	    stats.repo_stats.extra_files, stats.repo_stats.del_extra_files);
 	printf("VRP Entries: %u (%u unique)\n", stats.repo_tal_stats.vrps,
 	    stats.repo_tal_stats.vrps_uniqs);
 	printf("VAP Entries: %u (%u unique)\n", stats.repo_tal_stats.vaps,
@@ -1468,9 +1472,9 @@ usage:
 	fprintf(stderr,
 	    "usage: rpki-client [-ABcjmnoRrVv] [-b sourceaddr] [-d cachedir]"
 	    " [-e rsync_prog]\n"
-	    "                   [-H fqdn] [-S skiplist] [-s timeout] [-T table]"
-	    " [-t tal]\n"
-	    "                   [outputdir]\n"
+	    "                   [-H fqdn] [-P epoch] [-S skiplist] [-s timeout]"
+	    " [-T table]\n"
+	    "                   [-t tal] [outputdir]\n"
 	    "       rpki-client [-Vv] [-d cachedir] [-j] [-t tal] -f file ..."
 	    "\n");
 	return 1;
