@@ -1,4 +1,4 @@
-/*	$OpenBSD: mft.c,v 1.91 2023/04/26 16:32:41 claudio Exp $ */
+/*	$OpenBSD: mft.c,v 1.96 2023/06/29 10:28:25 tb Exp $ */
 /*
  * Copyright (c) 2022 Theo Buehler <tb@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -87,6 +87,8 @@ ASN1_SEQUENCE(Manifest) = {
 DECLARE_ASN1_FUNCTIONS(Manifest);
 IMPLEMENT_ASN1_FUNCTIONS(Manifest);
 
+#define GENTIME_LENGTH 15
+
 /*
  * Convert an ASN1_GENERALIZEDTIME to a struct tm.
  * Returns 1 on success, 0 on failure.
@@ -94,15 +96,18 @@ IMPLEMENT_ASN1_FUNCTIONS(Manifest);
 static int
 generalizedtime_to_tm(const ASN1_GENERALIZEDTIME *gtime, struct tm *tm)
 {
-	const char *data;
-	size_t len;
-
-	data = ASN1_STRING_get0_data(gtime);
-	len = ASN1_STRING_length(gtime);
+	/*
+	 * ASN1_GENERALIZEDTIME is another name for ASN1_STRING. Check type and
+	 * length, so we don't accidentally accept a UTCTime. Punt on checking
+	 * Zulu time for OpenSSL: we don't want to mess about with silly flags.
+	 */
+	if (ASN1_STRING_type(gtime) != V_ASN1_GENERALIZEDTIME)
+		return 0;
+	if (ASN1_STRING_length(gtime) != GENTIME_LENGTH)
+		return 0;
 
 	memset(tm, 0, sizeof(*tm));
-	return ASN1_time_parse(data, len, tm, V_ASN1_GENERALIZEDTIME) ==
-	    V_ASN1_GENERALIZEDTIME;
+	return ASN1_TIME_to_tm(gtime, tm);
 }
 
 /*
@@ -124,16 +129,15 @@ mft_parse_time(const ASN1_GENERALIZEDTIME *from,
 		return 0;
 	}
 
-	/* check that until is not before from */
-	if (ASN1_time_tm_cmp(&tm_until, &tm_from) < 0) {
-		warnx("%s: bad update interval", p->fn);
-		return 0;
-	}
-
 	if ((p->res->thisupdate = timegm(&tm_from)) == -1 ||
 	    (p->res->nextupdate = timegm(&tm_until)) == -1)
 		errx(1, "%s: timegm failed", p->fn);
 
+	if (p->res->thisupdate > p->res->nextupdate) {
+		warnx("%s: bad update interval", p->fn);
+		return 0;
+	}
+
 	return 1;
 }
 
@@ -277,12 +281,12 @@ mft_parse_econtent(const unsigned char *d, size_t dsz, struct parse *p)
 	int			 i, rc = 0;
 
 	if ((mft = d2i_Manifest(NULL, &d, dsz)) == NULL) {
-		cryptowarnx("%s: RFC 6486 section 4: failed to parse Manifest",
+		warnx("%s: RFC 6486 section 4: failed to parse Manifest",
 		    p->fn);
 		goto out;
 	}
 
-	if (!valid_econtent_version(p->fn, mft->version))
+	if (!valid_econtent_version(p->fn, mft->version, 0))
 		goto out;
 
 	p->res->seqnum = x509_convert_seqnum(p->fn, mft->manifestNumber);
@@ -349,6 +353,7 @@ struct mft *
 mft_parse(X509 **x509, const char *fn, const unsigned char *der, size_t len)
 {
 	struct parse	 p;
+	struct cert	*cert = NULL;
 	int		 rc = 0;
 	size_t		 cmsz;
 	unsigned char	*cms;
@@ -414,6 +419,9 @@ mft_parse(X509 **x509, const char *fn, const unsigned char *der, size_t len)
 	if (mft_parse_econtent(cms, cmsz, &p) == 0)
 		goto out;
 
+	if ((cert = cert_parse_ee_cert(fn, *x509)) == NULL)
+		goto out;
+
 	if (p.res->signtime > p.res->nextupdate) {
 		warnx("%s: dating issue: CMS signing-time after MFT nextUpdate",
 		    fn);
@@ -429,6 +437,7 @@ out:
 		*x509 = NULL;
 	}
 	free(crldp);
+	cert_free(cert);
 	free(cms);
 	return p.res;
 }
