@@ -1,4 +1,4 @@
-/*	$OpenBSD: output-json.c,v 1.36 2023/04/27 07:57:25 claudio Exp $ */
+/*	$OpenBSD: output-json.c,v 1.40 2023/06/26 18:39:53 job Exp $ */
 /*
  * Copyright (c) 2019 Claudio Jeker <claudio@openbsd.org>
  *
@@ -37,10 +37,10 @@ outputheader_json(struct stats *st)
 
 	gethostname(hn, sizeof hn);
 
-	json_do_object("metadata");
+	json_do_object("metadata", 0);
 
-	json_do_printf("buildmachine", "%s", hn);
-	json_do_printf("buildtime", "%s", tbuf);
+	json_do_string("buildmachine", hn);
+	json_do_string("buildtime", tbuf);
 	json_do_int("elapsedtime", st->elapsed_time.tv_sec);
 	json_do_int("usertime", st->user_time.tv_sec);
 	json_do_int("systemtime", st->system_time.tv_sec);
@@ -59,7 +59,7 @@ outputheader_json(struct stats *st)
 
 	json_do_array("talfiles");
 	for (i = 0; i < talsz; i++)
-		json_do_printf("name", "%s", tals[i]);
+		json_do_string("name", tals[i]);
 	json_do_end();
 
 	json_do_int("manifests", st->repo_tal_stats.mfts);
@@ -73,31 +73,27 @@ outputheader_json(struct stats *st)
 	json_do_int("vaps", st->repo_tal_stats.vaps);
 	json_do_int("uniquevaps", st->repo_tal_stats.vaps_uniqs);
 	json_do_int("cachedir_del_files", st->repo_stats.del_files);
-	json_do_int("cachedir_superfluous_files", st->repo_stats.extra_files);
 	json_do_int("cachedir_del_dirs", st->repo_stats.del_dirs);
+	json_do_int("cachedir_superfluous_files", st->repo_stats.extra_files);
+	json_do_int("cachedir_del_superfluous_files",
+	    st->repo_stats.del_extra_files);
 
 	json_do_end();
 }
 
 static void
-print_vap(struct vap *v, enum afi afi)
+print_vap(struct vap *v)
 {
 	size_t i;
-	int found = 0;
 
-	json_do_object("aspa");
+	json_do_object("aspa", 1);
 	json_do_int("customer_asid", v->custasid);
 	json_do_int("expires", v->expires);
 
 	json_do_array("providers");
-	for (i = 0; i < v->providersz; i++) {
-		if (v->providers[i].afi != 0 && v->providers[i].afi != afi)
-			continue;
-		found = 1;
-		json_do_int("provider", v->providers[i].as);
-	}
-	if (!found)
-		json_do_int("provider", 0);
+	for (i = 0; i < v->providersz; i++)
+		json_do_int("provider", v->providers[i]);
+
 	json_do_end();
 }
 
@@ -106,18 +102,9 @@ output_aspa(struct vap_tree *vaps)
 {
 	struct vap	*v;
 
-	json_do_object("provider_authorizations");
-
-	json_do_array("ipv4");
-	RB_FOREACH(v, vap_tree, vaps) {
-		print_vap(v, AFI_IPV4);
-	}
-	json_do_end();
-
-	json_do_array("ipv6");
-	RB_FOREACH(v, vap_tree, vaps) {
-		print_vap(v, AFI_IPV6);
-	}
+	json_do_array("aspas");
+	RB_FOREACH(v, vap_tree, vaps)
+		print_vap(v);
 	json_do_end();
 }
 
@@ -136,11 +123,11 @@ output_json(FILE *out, struct vrp_tree *vrps, struct brk_tree *brks,
 	RB_FOREACH(v, vrp_tree, vrps) {
 		ip_addr_print(&v->addr, v->afi, buf, sizeof(buf));
 
-		json_do_object("roa");
+		json_do_object("roa", 1);
 		json_do_int("asn", v->asid);
-		json_do_printf("prefix", "%s", buf);
+		json_do_string("prefix", buf);
 		json_do_int("maxLength", v->maxlength);
-		json_do_printf("ta", "%s", taldescs[v->talid]);
+		json_do_string("ta", taldescs[v->talid]);
 		json_do_int("expires", v->expires);
 		json_do_end();
 	}
@@ -148,11 +135,11 @@ output_json(FILE *out, struct vrp_tree *vrps, struct brk_tree *brks,
 
 	json_do_array("bgpsec_keys");
 	RB_FOREACH(b, brk_tree, brks) {
-		json_do_object("brks");
+		json_do_object("brks", 0);
 		json_do_int("asn", b->asid);
-		json_do_printf("ski", "%s", b->ski);
-		json_do_printf("pubkey", "%s", b->pubkey);
-		json_do_printf("ta", "%s", taldescs[b->talid]);
+		json_do_string("ski", b->ski);
+		json_do_string("pubkey", b->pubkey);
+		json_do_string("ta", taldescs[b->talid]);
 		json_do_int("expires", b->expires);
 		json_do_end();
 	}
